
class Direct:
    def __init__(self, j=0.0, d=0.0):
        self.type = 'direct'
        self.j = j
        self.d_in = d


class Coupler:
    def __init__(self, j=0.0, d_in=0.0, d_out=0.0):
        self.type = 'coupler'
        self.j = j
        self.d_in = d_in
        self.d_out = d_out


class Wheel:
    def __init__(self, diameter, j=0.0):
        self.type = 'wheel'
        self.diameter = diameter
        self.j = j
        self.d


class Screw:
    
    def __init__(self, pitch=None, lead=None, j=0.0):
        
        self.type = 'screw'
        
        if (pitch is None and lead is None) or (pitch is not None and lead is not None):
            raise ValueError("Either lead or pitch must be a float.")
        else:
            if pitch is not None:
                self.pitch = pitch
                self.lead = 1/pitch
            else: # lead is not None
                self.lead = lead
                self.pitch = 1/lead
        
        self.j = j


class Gear:
    def __init__(self, ratio, j_in=0.0, j_out=0.0, d_in=0.0, d_out=0.0):
        self.type = 'gear'
        self.ratio = ratio
        self.j_in = j_in
        self.j_out = j_out
        self.d_in = d_in
        self.d_out = d_out




from typing import List
import pandas as pd

import pymotor.files as files
import pymotor.plots as plots
from pymotor.conversions import *


class Motor:
    '''Motor objects describe a physical motor. 

    Motor.j is the moment of interia in kg*m^2.

    Motor.curve is a pandas DataFrame containing points representing the
        torque versus speed curve. 

    Motor.curve['hz'] are the angular velocities in Hz.

    Motor.curve['tau'] are the torques in N*m.

    Motor.tau(hz) returns an interpolated torque value in N*m.

    Motor.hz_min and Motor.hz_max are created at init and define the range
        for the tau method's hz argument. 

    Motor.plot(filename) creates a PNG plot of the torque curve. If the
        filename argument is not included, the plot will attempt to display
        on screen.

    Motor.d is the spindle diameter in m. 

    The following descriptive attributes are provided:
        Motor.name
        Motor.manufacturer
        Motor.description
    '''
    def __init__(self,
        name: str ='PL23HSAP4150 60VDC 1.5A',
        manufacturer: str = 'Moons',
        description: str = 'Stepper NEMA23',
        curve_hz: List[float] = [0, rpm(300), rpm(600), rpm(900), rpm(1200), rpm(1500), rpm(1800)],
        curve_tau: List[float] = [2.5, 2.2, 1.3, 0.9, 0.7, 0.6, 0.5],
        j: float = gcm2(460),
        d_out: float = inch(0.25),
        ):

        self.name = str(name)
        self.manufacturer = str(manufacturer)
        self.description = str(description)

        if self._j_ok(j):
            self.j = j
        else:
            raise ValueError("j (kg*m^2) cannot be negative.")

        if self._d_out_ok(d_out):
            self.d_out = d_out
        else:
            raise ValueError("d_out (m) must be positive.")

        if self._curve_hz_ok(curve_hz) and self._curve_tau_ok(curve_tau):
            if len(curve_hz) == len(curve_tau):
                self.curve = pd.DataFrame(data={'hz': curve_hz, 'tau': curve_tau})
                self.hz_min = self.curve['hz'].min()
                self.hz_max = self.curve['hz'].max()
            else:
                raise ValueError("curve_hz and curve_tau lists must be the same length.")        
        else:
            raise ValueError("curve_hz (Hz) and curve_tau (N*m) values must be positive. curve_hz values must be ascending.")

            
    def tau(self, hz: float) -> float:
        '''Given speed (Hz) returns interpolated tau (N*m).'''
        if self._hz_range_ok(hz):
            for i in range(self.curve['hz'].size):
                if hz <= self.curve['hz'].iloc[i + 1]:
                    hz1 = self.curve['hz'].iloc[i]
                    hz2 = self.curve['hz'].iloc[i + 1]
                    tau1 = self.curve['tau'].iloc[i]
                    tau2 = self.curve['tau'].iloc[i + 1]
                    break
            return (((hz - hz1) * (tau2 - tau1)) / (hz2 - hz1)) + tau1
        else:
            raise ValueError("hz must be between Motor.hz_min and Motor.hz_max.")


    def plot(self, 
        filename: str = None,
        plot_title: str = 'Motor Torque Curve',
        hz_label: str = 'Angular Velocity (Hz)',
        tau_label: str = 'Torque (N*m)',
        ):
        '''Plots torque curve data to PNG image, or screen if filename is not specified.'''
        df = self.curve[['hz', 'tau']]
        df = df.rename(columns={'hz': hz_label, 'tau': tau_label})
        plots._plot_df(df, plot_title=plot_title, filename=filename, height=3, width=5)


    def save(self, filename: str):
        '''Save Motor object data to file.'''
        save_data = (
            self.j,
            self.curve,
            self.hz_min,
            self.hz_max,
            self.d_out,
            self.name,
            self.manufacturer,
            self.description,
        )        
        files._save(save_data, filename)


    def load(self, filename: str):
        '''Load Motor object data from file.'''
        load_data = files._load(filename)
        (
            self.j,
            self.curve,
            self.hz_min,
            self.hz_max,
            self.d_out,
            self.name,
            self.manufacturer,
            self.description,
        ) = load_data


    def _j_ok(self, j: float) -> bool:
        '''True if j positive.'''
        if j < 0.0:
            return False
        return True


    def _d_out_ok(self, d_out: float) -> bool:
        '''True if j positive.'''
        if d_out <= 0.0:
            return False
        return True


    def _curve_hz_ok(self, curve_hz: List[float]) -> bool:
        '''True if values are ascending and positive.'''
        for i in range(len(curve_hz)):
            if curve_hz[i] < 0.0:
                return False
            if i > 0:
                if curve_hz[i] <= curve_hz[i - 1]:
                    return False
        return True


    def _curve_tau_ok(self, curve_hz: List[float]) -> bool:
        '''True if values are positive.'''
        for i in range(len(curve_hz)):
            if curve_hz[i] < 0.0:
                return False
        return True


    def _hz_range_ok(self, hz: float) -> bool:
        '''True if hz between Motor.hz_min and Motor.hz_max.'''
        if hz < self.hz_min or hz > self.hz_max:
            return False
        return True




import numpy as np

# Pi Constants

TWO_PI = 2.0 * np.pi
ONE_OVR_TWO_PI = 1.0 / TWO_PI

# Time Conversion Constants

SEC_PER_MIN = 60.0
MIN_PER_SEC = 1.0 / SEC_PER_MIN

MIN_PER_HOUR = 60.0
HOUR_PER_MIN = 1.0 / MIN_PER_HOUR

SEC_PER_HOUR = SEC_PER_MIN * MIN_PER_HOUR
HOUR_PER_SEC = 1.0 / SEC_PER_HOUR  

# Length Conversion Constants

M_PER_INCH = 0.0254
INCH_PER_M = 1.0 / M_PER_INCH
 
MM_PER_M = 1000.0
M_PER_MM = 1.0 / MM_PER_M

CM_PER_M = 100.0
M_PER_CM = 1.0 / CM_PER_M

# Velocity Conversion Constants

MPS_PER_IPM = MIN_PER_SEC * M_PER_INCH
IPM_PER_MPS = SEC_PER_MIN * INCH_PER_M

# Area Conversion Constants

CM2_PER_M2 = CM_PER_M * CM_PER_M
M2_PER_CM2 = 1.0 / CM2_PER_M2

MM2_PER_M2 = MM_PER_M * MM_PER_M
M2_PER_MM2 = 1.0 / MM2_PER_M2

# Mass Conversion Constants

G_PER_KG = 1000.0
KG_PER_G = 1.0 / G_PER_KG

# Moment of Inertia Conversion Constants

KGM2_PER_GCM2 = KG_PER_G * M2_PER_CM2


def inch(x_inch: float) -> float:
    '''Converts from inch to meters.'''
    return x_inch * M_PER_INCH

def to_inch(x_m: float) -> float:
    '''Converts from meters to inch.'''
    return x_m * INCH_PER_M

def ipm(v_ipm: float) -> float:
    '''Converts from IPM to m/s.'''
    return v_ipm * MPS_PER_IPM

def to_ipm(v_mps: float) -> float:
    '''Converts from m/s to IPM'''
    return v_mps * IPM_PER_MPS

def radps(omega_radps: float) -> float:
    '''Converts from Rads/s to Hz.'''
    return omega_radps * ONE_OVR_TWO_PI

def to_radps(omega_hz: float) -> float:
    '''Converts from Hz to Rads/s.'''
    return omega_hz * TWO_PI

def rpm(omega_rpm: float) -> float:
    '''Converts from RPM to Hz.'''
    return omega_rpm * MIN_PER_SEC

def to_rpm(omega_hz: float) -> float:
    '''Converts from Hz to RPM.'''
    return omega_hz * SEC_PER_MIN

def radps2(alpha_radps2: float) -> float:
    '''Converts from Rads/s^2 to Hz/s.'''
    return alpha_radps2 * ONE_OVR_TWO_PI

def to_radps2(alpha_hzps: float) -> float:
    '''Converts from Hz/s to Rads/s^2.'''
    return alpha_hzps * TWO_PI

def rpmps(alpha_rpmps: float) -> float:
    '''Converts from RPM/s to Hz/s.'''
    return alpha_rpmps * MIN_PER_SEC

def to_rpmps(alpha_hzps: float) -> float:
    '''Converts from Hz/s to RPM/s.'''
    return alpha_hzps * SEC_PER_MIN

def gcm2(j_gcm2: float) -> float:
    '''Converts from g*cm^2 to kg*m^2.'''
    return j_gcm2 * KGM2_PER_GCM2

def to_gcm2(j_kgm2: float) -> float:
    '''Converts from kg*m^2 to g*cm^2.'''
    return j_kgm2 * GCM2_PER_KGM2




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pymotor",
    version="0.3.0",
    install_requires=[
            'matplotlib==2.2.2',
            'numpy==1.15.0',
            'pandas==0.22.0',
            'scipy==1.1.0',
      ],
    author="Randy Rubin",
    author_email="randymrubin@gmail.com",
    description="Generates motion and force profiles.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rmrubin/pymotor",
    packages=setuptools.find_packages(),
    classifiers=(
        "Development Status :: 3 - Alpha",
        "Topic :: Scientific/Engineering :: Physics",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)




import matplotlib.pyplot as plt

DEFAULT_PLOT_WIDTH_INCHES = 6.5
DEFAULT_PLOT_HEIGHT_INCHES = 9.0

def _plot_df(df,
    plot_title='pandas.DataFrame',
    filename=None,
    width=DEFAULT_PLOT_WIDTH_INCHES,
    height=DEFAULT_PLOT_HEIGHT_INCHES,
    ):

    if filename is None:
        plt.switch_backend('TKAgg')
    else:
        plt.switch_backend('Agg')

    labels = list(df.columns.values)
    num_plots = df.shape[1] - 1   

    plt.figure(figsize=(width, height), clear=True)
 
    for i in range(num_plots):

        plt.subplot(num_plots, 1, i + 1)
        plt.plot(df.iloc[:, 0].get_values(), df.iloc[:, i + 1].get_values(),
            linestyle='solid', linewidth=1, color=(0.0, 0.0, 0.0))
        plt.grid(linestyle=':', linewidth=1, color=(0.75, 0.75, 0.75))
        plt.ylabel(labels[i + 1])

        if i == 0:
            plt.title(plot_title)
        if i == num_plots - 1:
            plt.xlabel(labels[0])
    
    plt.tight_layout()

    if filename is None:
        plt.show()
    else:
        plt.savefig(filename)

def _plot_df_dual(df, series,
    plot_title='pandas.DataFrame',
    filename=None,
    width=DEFAULT_PLOT_WIDTH_INCHES,
    height=DEFAULT_PLOT_HEIGHT_INCHES,
    ):

    if filename is None:
        plt.switch_backend('TKAgg')
    else:
        plt.switch_backend('Agg')

    labels = list(df.columns.values)
    num_plots = df.shape[1] - 1   

    plt.figure(figsize=(width, height), clear=True)
 
    for i in range(num_plots):

        plt.subplot(num_plots, 1, i + 1)
        plt.plot(df.iloc[:, 0].get_values(), df.iloc[:, i + 1].get_values(),
            linestyle='solid', linewidth=1, color=(0.0, 0.0, 0.0))
        if i == 0:
            plt.plot(df.iloc[:, 0].get_values(), series.get_values(),
                linestyle='--', linewidth=1, color=(1.0, 0.0, 0.0))
        plt.grid(linestyle=':', linewidth=1, color=(0.75, 0.75, 0.75))
        plt.ylabel(labels[i + 1])

        if i == 0:
            plt.title(plot_title)
        if i == num_plots - 1:
            plt.xlabel(labels[0])
    
    plt.tight_layout()

    if filename is None:
        plt.show()
    else:
        plt.savefig(filename)




import pandas as pd
import pickle

def _save(save_data, filename):
    with open(filename, 'wb') as f:  
        pickle.dump(save_data, f)    

def _load(filename):
    with open(filename, 'rb') as f:
        return pickle.load(f)    

def _html(df, filename):
    with open(filename, 'w') as f:
        print(df.to_html(), file=f)

def _csv(df, filename):
    with open(filename, 'w') as f:
        print(df.to_csv(), file=f)

def _xlsx(df, filename):
    writer = pd.ExcelWriter(filename)
    df.to_excel(writer, 'Sheet1')
    writer.save()

def _txt(text_data, filename):
    with open(filename, 'w') as f:
        print(text_data, file=f)




import numpy as np
from scipy import signal
import pandas as pd

import pymotor.files as files
import pymotor.plots as plots
from pymotor.conversions import *

class Profile:

    def _gen_deriv(self, data, fs, init=0):

        tablen = len(data)
        deriv_tab = np.empty(tablen)
        last = np.float(init)

        i = 0
        for sample in data:
            deriv_tab[i] = np.float((sample - last) * fs)
            last = np.float(sample)
            i += 1

        return deriv_tab

    def _gen_intgrl(self, data, fs, init=0):

        tablen = len(data)
        intgrl_tab = np.empty(tablen)
        intgrl_val = np.float(init)

        i = 0
        for sample in data:
            intgrl_val += np.float(sample / fs)
            intgrl_tab[i] = intgrl_val
            i += 1

        return intgrl_tab

    def save(self, filename):
        files._save((self.settings, self.stats, self.profile), filename)    

    def load(self, filename):
        (self.settings, self.stats, self.profile) = files._load(filename)    

    def html(self, filename):
        files._html(self.profile, filename)

    def csv(self, filename):
        files._csv(self.profile, filename)

    def xlsx(self, filename):
        files._xlsx(self.profile, filename)

    def print(self, filename=None):

        profile_str = "\n[i] Profile Data Table\n\n"
        profile_str += self.profile.to_string() + '\n'     

        settings_str = "\n[i] Profile Settings\n\n"
        for key, value in self.settings.items():
            settings_str += key + ': ' + str(value) + '\n'
    
        stats_str = "\n[i] Profile Stats\n\n"
        for key, value in self.stats.items():
            stats_str += key + ': ' + str(value) + '\n'
                
        if filename:
            files._txt((settings_str + stats_str + profile_str), filename)
        else:
            print(profile_str + settings_str + stats_str)

    def drop_profile(self):
        del self.profile


class LinearMotion(Profile):

    def __init__(self,
        settings=None,
        fs = 1000,
        max_velocity = 1,
        acc_mode = 'time',
        acc_value = 1,
        acc_smooth = True,
        con_mode = 'time',
        con_value = 1,
        dec_mode = 'time',
        dec_value = 1,
        dec_smooth = True,
        ):

        if settings is None:
            self.settings = {
                'fs': fs,
                'max_velocity': max_velocity,
                'acc_mode': acc_mode,
                'acc_value': acc_value,
                'acc_smooth': acc_smooth,
                'con_mode': con_mode,
                'con_value': con_value,
                'dec_mode': dec_mode,
                'dec_value': dec_value,
                'dec_smooth': dec_smooth,
                }
        else:
            self.settings = settings

        self.generate()

    def generate(self):
        (self.profile, self.stats) = self._gen_linpro(self.settings)

    def plot(self, 
        filename=None,
        plot_title='Linear Motion Profile',
        t_label='Time (s)',
        x_label='Distance (m)',
        v_label='Velocity (m/s)',
        a_label='Acceleration (m/s^2)'
        ):

        df = self.profile[['t', 'a', 'v', 'x']]
        df = df.rename(columns={'t': t_label, 'a': a_label, 'v': v_label, 'x': x_label})
        plots._plot_df(df, plot_title=plot_title, filename=filename)

    def _gen_x_from_v(self, v, fs, x0=0):
        return self._gen_intgrl(v, fs, x0)

    def _gen_v_from_a(self, a, fs, v0=0):
        return self._gen_intgrl(a, fs, v0)

    def _gen_v_from_x(self, x, fs, x0=0):
        return self._gen_deriv(x, fs, x0)

    def _gen_a_from_v(self, v, fs, v0=0):
        return self._gen_deriv(v, fs, v0)

    def _gen_t_from_v(self, v, fs):
        t = np.arange(len(v), dtype='float')
        t = t / fs
        return t

    def _get_t1_from_v1_and_x1(self, v1, x1):
        return (2 / v1) * x1

    def _get_t1_from_v1_and_a1(self, v1, a1):
        return (1 / a1) * v1

    def _gen_acc_from_v_and_t(self, v1, t1, fs, smooth):

        tablen = np.int(t1 * fs)

        if smooth is True:
            v = signal.get_window('hann', tablen * 2)
        else:
            v = signal.get_window('triang', tablen * 2)

        v = v[:tablen]
        v = v * v1
        x = self._gen_x_from_v(v, fs)
        a = self._gen_a_from_v(v, fs)
        t = self._gen_t_from_v(v, fs)

        profile = pd.DataFrame({'t': t, 'x': x, 'v': v, 'a': a})

        return profile

    def _gen_con_from_v_and_t(self, v1, t1, x0, fs):

        v1 = np.float(v1)
        t1 = np.float(t1)
        tablen = np.int(t1 * fs)

        v = np.empty(tablen, dtype='float')
        v.fill(v1)
        
        x = self._gen_x_from_v(v, fs=fs, x0=x0)
        a = np.zeros(tablen, dtype='float')
        t = self._gen_t_from_v(v, fs)
             
        profile = pd.DataFrame({'t': t, 'x': x, 'v': v, 'a': a})

        return profile

    def _gen_dec_from_v_and_t(self, v1, t1, v0, x0, fs, smooth):

        tablen = np.int(t1 * fs)

        if smooth is True:
            v = signal.get_window('hann', tablen * 2)
        else:
            v = signal.get_window('triang', tablen * 2)

        v = v[tablen:]
        v = v * v1

        x = self._gen_x_from_v(v, fs, x0)
        a = self._gen_a_from_v(v, fs, v0)
        t = self._gen_t_from_v(v, fs)
             
        profile = pd.DataFrame({'t': t, 'x': x, 'v': v, 'a': a})

        return profile

    def _gen_linpro(self, settings):

        fs = settings['fs']
        max_velocity = settings['max_velocity']
        acc_mode = settings['acc_mode']
        acc_value = settings['acc_value']
        con_mode = settings['con_mode']
        con_value = settings['con_value']
        dec_mode = settings['dec_mode']
        dec_value = settings['dec_value']

        if settings['acc_smooth'] is None or settings['acc_smooth'] is False:
            acc_smooth = False
        else:
            acc_smooth = True
        
        if settings['dec_smooth'] is None or settings['dec_smooth'] is False:
            dec_smooth = False
        else:
            dec_smooth = True

        if acc_mode == 'distance':
            acc_t1 = self._get_t1_from_v1_and_x1(max_velocity, acc_value)
        elif acc_mode == 'acceleration':
            acc_t1 = self._get_t1_from_v1_and_a1(max_velocity, acc_value)
        elif acc_mode == 'time':
            acc_t1 = acc_value
        else:
            raise ValueError("Acceptable input for acc_mode is 'distance', 'time', or 'acceleration'.")    

        if con_mode == 'distance':
            con_t1 = self._get_t1_from_v1_and_x1(max_velocity, con_value)
        elif con_mode == 'time':
            con_t1 = con_value
        else:
            raise ValueError("Acceptable input for con_mode is 'distance' or 'time'.")

        if dec_mode == 'distance':
            dec_t1 = self._get_t1_from_v1_and_x1(max_velocity, dec_value)
        elif dec_mode == 'acceleration':
            dec_t1 = self._get_t1_from_v1_and_a1(max_velocity, dec_value)
        elif dec_mode == 'time':
            dec_t1 = dec_value
        else:
            raise ValueError("Acceptable input for dec_mode is 'distance', 'time', or 'acceleration'.")

        acc_profile = self._gen_acc_from_v_and_t(v1=max_velocity, t1=acc_t1, smooth=acc_smooth, fs=fs)
        con_profile = self._gen_con_from_v_and_t(v1=max_velocity, t1=con_t1, x0=acc_profile['x'].iloc[-1], fs=fs)
        dec_profile = self._gen_dec_from_v_and_t(v1=max_velocity, t1=dec_t1, v0=con_profile['v'].iloc[-1], x0=con_profile['x'].iloc[-1], smooth=dec_smooth, fs=fs)
        
        profile = pd.concat([acc_profile, con_profile, dec_profile], ignore_index=True)
        
        t = np.arange(profile['x'].size)
        t = t / fs
        t = pd.Series(t)
        profile['t'] = t

        stats = {
            'acc_size': acc_profile['x'].size,
            'acc_a_max': acc_profile['a'].max(),
            'acc_a_mean': acc_profile['a'].mean(),
            'acc_t': acc_profile['x'].size / fs,
            'acc_x': acc_profile['x'].ptp(),
            'con_size': con_profile['x'].size,
            'con_t': con_profile['x'].size / fs,
            'con_x': con_profile['x'].ptp(),
            'dec_size': dec_profile['x'].size,
            'dec_a_min': dec_profile['a'].min(),
            'dec_a_mean': dec_profile['a'].mean(),
            'dec_t': dec_profile['x'].size / fs,
            'dec_x': dec_profile['x'].ptp(),
            }

        return (profile, stats)


class LinearForce(Profile):

    def __init__(self, settings, linear_motion_object):

        self.settings = settings
        self.lm = linear_motion_object
        self.generate()

    def generate(self):

        self.profile = self.lm.profile
        self.lm.drop_profile()
        
        self.stats = {}
        self._calc_force_constants()

        f_series = self.profile['a'].copy()
        f_series = f_series.apply(self._get_force)
        self.profile['f'] = f_series

    def plot(self, 
        filename=None,
        plot_title='Required Force',
        t_label='Time (s)',
        v_label='Velocity (m/s)',
        f_label='Force (N)',
        ):

        df = self.profile[['t', 'f', 'v']]
        df = df.rename(columns={'t': t_label, 'f': f_label, 'v': v_label})
        plots._plot_df(df, plot_title=plot_title, filename=filename, height=6.0)

    def _calc_force_constants(self):

        moving_mass = self.settings['moving_mass']
        f_preload = self.settings['preload_force']
        incline = self.settings['incline_angle']
        friction_coef = self.settings['friction_coef']
        gravity = self.settings['gravity']
        sf_and_eff = self.settings['safety_factor'] / self.settings['efficiency']
        
        f_incline = moving_mass * gravity * np.sin(np.radians(incline))
        f_friction = friction_coef * moving_mass * gravity * np.cos(np.radians(incline))
        f_constant = f_preload + f_incline + f_friction

        self._f_scale = sf_and_eff * moving_mass 
        self._f_offset = sf_and_eff * f_constant

        self.stats['f_incline'] = f_incline
        self.stats['f_friction'] = f_friction
        self.stats['f_constant'] = f_constant

    def _get_force(self, a):
        return a * self._f_scale + self._f_offset


class AngularTorque(Profile):

    def __init__(self, linear_force_object, motor, coupler, gear, drivetrain):
        self.lf = linear_force_object
        self.motor = motor
        self.coupler = coupler
        self.gear = gear
        self.drivetrain = drivetrain
        self.generate()

    def generate(self):
        
        self.profile = self.lf.profile
        self.lf.drop_profile()
        
        self.settings = {}
        self.settings['safety_factor'] = self.lf.settings['safety_factor']

        self.stats = {}
        self._calc_intertia()

        revs_series = self.profile['x'].copy()
        self.profile['revs'] = revs_series.apply(self._get_revs_from_x)

        hz_series = self.profile['v'].copy()
        self.profile['hz'] = hz_series.apply(self._get_hz_from_v)
        
        hzps_series = self.profile['a'].copy()
        self.profile['hzps'] = hzps_series.apply(self._get_hzps_from_a)

        tau_rotating_series = self.profile['hzps'].copy()        
        self.profile['tau_rotating'] = tau_rotating_series.apply(self._get_tau_rotating_from_hzps)

        tau_linear_series = self.profile['f'].copy()        
        self.profile['tau_linear'] = tau_linear_series.apply(self._get_tau_linear_from_f)
        
        self.profile['tau'] = self.profile['tau_rotating'] + self.profile['tau_linear']

        tau_motor_series = self.profile['hz'].copy()
        self.profile['tau_motor'] = tau_motor_series.apply(self._get_tau_motor_from_hz)

    def plot(self, 
        filename=None,
        plot_title='Required (Black) and Available (Red) Torque',
        t_label='Time (s)',
        hz_label='Velocity (Hz)',
        tau_label='Torque (N*m)',
        ):

        df = self.profile[['t', 'tau', 'hz']]
        df = df.rename(columns={'t': t_label, 'tau': tau_label, 'hz': hz_label})
        series = self.profile['tau_motor']
        plots._plot_df_dual(df, series, plot_title=plot_title, filename=filename, height=6.0)

    def _calc_intertia(self):

        j_linear = self.lf.settings['moving_mass'] / (2 * np.pi * self.drivetrain.pitch)**2
        j_in = self.coupler.j + self.gear.j_in
        j_out = self.gear.j_out + self.drivetrain.j + j_linear
        j_load = j_in + j_out * self.gear.ratio**2
        j_ratio = j_load / self.motor.j
        j_rotating = self.motor.j + j_in + (self.gear.j_out + self.drivetrain.j) * self.gear.ratio**2

        self._tau_rotating_scale = 2.0 * np.pi * j_rotating * self.settings['safety_factor']
        self._tau_linear_scale = (self.drivetrain.lead * self.gear.ratio) / (2.0 * np.pi)
        self._xva_scale = self.drivetrain.pitch / self.gear.ratio

        self.stats['drivetrain_type'] = self.drivetrain.type
        self.stats['gear_ratio'] = self.gear.ratio
        self.stats['j_motor'] = self.motor.j
        self.stats['j_coupler'] = self.coupler.j
        self.stats['j_gear_in'] = self.gear.j_in
        self.stats['j_gear_out'] = self.gear.j_out
        self.stats['j_drivetrain'] = self.drivetrain.j
        self.stats['j_linear'] = j_linear
        self.stats['j_in'] = j_in
        self.stats['j_out'] = j_out
        self.stats['j_load'] = j_load
        self.stats['j_ratio'] = j_ratio
        self.stats['j_rotating'] = j_rotating

    def _get_revs_from_x(self, x):
        return x * self._xva_scale
    
    def _get_hz_from_v(self, v):
        return v * self._xva_scale

    def _get_hzps_from_a(self, a):
        return a * self._xva_scale

    def _get_tau_rotating_from_hzps(self, hzps):
        return hzps * self._tau_rotating_scale

    def _get_tau_linear_from_f(self, f):
        return f * self._tau_linear_scale

    def _get_tau_motor_from_hz(self, hz):
        return self.motor.tau(hz) 
